<?php

namespace Opencart\Admin\Controller\Extension\Xamax\Payment;

require_once DIR_EXTENSION . 'xamax/system/library/init.php';

use Opencart\System\Engine\Controller;
use Xamax\XamaxClient;

class Xamax extends Controller
{
	private $error = array();

	public function index()
	{
		$this->load->language('extension/xamax/payment/xamax');
		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');
		$this->load->model('localisation/order_status');
		$this->load->model('localisation/geo_zone');

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			$this->model_setting_setting->editSetting('payment_xamax', $this->request->post);
			$this->session->data['success'] = $this->language->get('text_success');
			$this->response->redirect($this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true));
		}

		$data['order_statuses']     = $this->model_localisation_order_status->getOrderStatuses();
		$data['geo_zones']          = $this->model_localisation_geo_zone->getGeoZones();
		$data['receive_currencies'] = array('BTC', 'USDT', 'DO_NOT_CONVERT');

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		$data['save'] = $this->url->link('extension/xamax/payment/xamax.save', 'user_token=' . $this->session->data['user_token']);
		$data['back'] = $this->url->link('marketplace/extension?user_token=' . $this->session->data['user_token'] . '&type=payment');

		$data['breadcrumbs'] = array();
		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'], true)
		);
		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_extension'),
			'href' => $this->url->link('marketplace/extension', 'user_token=' . $this->session->data['user_token'] . '&type=payment', true)
		);
		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link('extension/xamax/payment/xamax', 'user_token=' . $this->session->data['user_token'], true)
		);

		$fields = array('payment_xamax_status', 'payment_xamax_api_access_token', 'payment_xamax_api_refresh_token', 'payment_xamax_api_secret',
			'payment_xamax_order_status_id', 'payment_xamax_pending_status_id', 'payment_xamax_confirming_status_id', 'payment_xamax_processing_status_id',
			'payment_xamax_invalid_status_id', 'payment_xamax_expired_status_id', 'payment_xamax_canceled_status_id', 'payment_xamax_refunded_status_id',
			'payment_xamax_total', 'payment_xamax_geo_zone_id', 'payment_xamax_test_mode');


		foreach ($fields as $field) {
			if (isset($this->request->post[$field])) {
				$data[$field] = $this->request->post[$field];
			} else {
				$data[$field] = $this->config->get($field);
			}
		}

		$data['payment_xamax_sort_order'] = isset($this->request->post['payment_xamax_sort_order']) ?
			$this->request->post['payment_xamax_sort_order'] :  $this->config->get('payment_xamax_sort_order');


		if (empty($data['payment_xamax_api_access_token']) && !empty($data['payment_xamax_api_secret']))
			$data['payment_xamax_api_access_token'] = $data['payment_xamax_api_secret'];

		$data['header'] = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer'] = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view('extension/xamax/payment/xamax', $data));
	}

	/**
	 * save method
	 *
	 * @return void
	 */
	public function save()
	{
		// loading example payment language
		$this->load->language('extension/xamax/payment/xamax');
		$json = [];
		// checking file modification permission
		if (!$this->user->hasPermission('modify', 'extension/xamax/payment/xamax')) {
			$json['error']['warning'] = $this->language->get('error_permission');
		}
		if (!$json) {
			$this->load->model('setting/setting');
			$this->model_setting_setting->editSetting('payment_xamax', $this->request->post);
			$json['success'] = $this->language->get('text_success');
		}
		$this->response->addHeader('Content-Type: application/json');
		$this->response->setOutput(json_encode($json));
	}

	public function install()
	{
		if ($this->user->hasPermission('modify', 'extension/payment')) {
			$this->load->model('extension/xamax/payment/xamax');

			$this->model_extension_xamax_payment_xamax->install();
		}
	}

	public function uninstall()
	{
		if ($this->user->hasPermission('modify', 'extension/payment')) {
			$this->load->model('extension/xamax/payment/xamax');

			$this->model_extension_xamax_payment_xamax->uninstall();
		}
	}

	protected function validate()
	{
		if (!$this->user->hasPermission('modify', 'extension/xamax/payment/xamax')) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (!$this->error) {
			$testConnection = XamaxClient::testConnection(array(
				'environment' => $this->request->post['payment_xamax_test_mode'] == 1 ? 'sandbox' : 'live',
				'refresh_token' => $this->request->post['payment_xamax_api_refresh_token']));

			if (!isset($testConnection['refresh_token'])) {
				$this->error['warning'] = $testConnection;
			} else {
				$this->model_setting_setting->editSetting('payment_xamax', ['payment_xamax_api_refresh_token' => $testConnection['refresh_token']]);
			}
		}

		return !$this->error;
	}
}
