<?php

namespace Opencart\Catalog\Controller\Extension\Xamax\Payment;

use Opencart\System\Engine\Controller;
use QRcode;
use Xamax\Merchant\Order;
use Xamax\XamaxClient;

require_once DIR_EXTENSION . 'xamax/system/library/init.php';
require_once DIR_EXTENSION . 'xamax/system/library/version.php';

class Xamax extends Controller
{
    public function index()
    {
        $this->load->language('extension/xamax/payment/xamax');
        $this->load->model('checkout/order');

        $data['chaingTypes'] = [];
        $ct_order = Order::chaingtype();

        if (is_array($ct_order->blockchains) && count($ct_order->blockchains) > 0) {
            $payloads = [];

            foreach ($ct_order->blockchains as $currency) {
                if ($currency['code'] != 'bsc') {
                    $payloads[] = $this->getIconAndLabelForCurrency($currency['code']) + ['currency' => $currency['code']];
                }
                if (isset($currency['tokens'])) {
                    foreach ($currency['tokens'] as $token) {
                        $payloads[] = $this->getIconAndLabelForCurrency($token['code']) + ['currency' => $token['code']];
                    }
                }
            }

            usort($payloads, function ($a, $b) {
                return $a['priority'] - $b['priority'];
            });

            $data['chaingTypes'] = $payloads;
        }

        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['action'] = $this->url->link('extension/xamax/payment/xamax.checkout', '', true);

        return $this->load->view('extension/xamax/payment/xamax', $data);
    }

    protected function getIconAndLabelForCurrency(string $currency): array
    {
        switch (strtolower($currency)) {
            case 'usdt':
                return [
                    'network' => 'ETHEREUM',
                    'priority' => 0,
                    'label' => 'USDT (ERC-20)',
                    'icon' => '/extension/xamax/catalog/view/image/usdt_erc20.svg'
                ];
            case 'usdt_trc20':
                return [
                    'network' => 'TRON',
                    'priority' => 2,
                    'label' => 'USDT (TRC-20)',
                    'icon' => '/extension/xamax/catalog/view/image/usdt_trc20.svg'
                ];
            case 'btc':
                return [
                    'network' => 'BITCOIN',
                    'priority' => 3,
                    'label' => 'BITCOIN',
                    'icon' => '/extension/xamax/catalog/view/image/btc.svg'
                ];
            case 'eth':
                return [
                    'network' => 'ETHEREUM',
                    'priority' => 4,
                    'label' => 'ETHEREUM',
                    'icon' => '/extension/xamax/catalog/view/image/eth.svg'
                ];
            case 'bscusd':
                return [
                    'network' => 'BINANCE SMART CHAIN',
                    'priority' => 5,
                    'label' => 'USDT (BEP-20)',
                    'icon' => '/extension/xamax/catalog/view/image/usdt_bep20.svg'
                ];
            case 'busd':
                return [
                    'network' => 'Binance Smart Chain',
                    'priority' => 6,
                    'label' => 'BUSD (BEP-20)',
                    'icon' => '/extension/xamax/catalog/view/image/busd_bep20.svg'
                ];
            case 'bnb':
                return [
                    'network' => 'Binance Smart Chain',
                    'priority' => 7,
                    'label' => 'BINANCE COIN',
                    'icon' => '/extension/xamax/catalog/view/image/bnb.svg'
                ];
            case 'usdc':
                return [
                    'network' => 'ETHEREUM',
                    'priority' => 8,
                    'label' => 'USDC (ERC-20)',
                    'icon' => '/extension/xamax/catalog/view/image/usdc_erc20.svg'
                ];
            case 'bch':
                return [
                    'network' => 'BITCOIN CASH',
                    'priority' => 9,
                    'label' => 'BITCOIN CASH',
                    'icon' => '/extension/xamax/catalog/view/image/bch.svg'
                ];
            case 'ltc':
                return [
                    'network' => 'LITECOIN',
                    'priority' => 10,
                    'label' => 'LITECOIN',
                    'icon' => '/extension/xamax/catalog/view/image/ltc.svg'
                ];
            case 'qtum':
                return [
                    'network' => 'QTUM',
                    'priority' => 11,
                    'label' => 'QTUM',
                    'icon' => '/extension/xamax/catalog/view/image/qtum.svg'
                ];
            case 'zec':
                return [
                    'network' => 'ZEC',
                    'priority' => 12,
                    'label' => 'ZEC',
                    'icon' => '/extension/xamax/catalog/view/image/zec.svg'
                ];
            case 'trx':
                return [
                    'network' => 'TRON',
                    'priority' => 13,
                    'label' => 'TRON',
                    'icon' => '/extension/xamax/catalog/view/image/trx.svg'
                ];
            case 'bsc':
                return [
                    'network' => 'BINANCE SMART CHAIN',
                    'priority' => 14,
                    'label' => 'BINANCE SMART CHAIN',
                    'icon' => '/extension/xamax/catalog/view/image/bsc.svg'
                ];
            case 'btg':
                return [
                    'network' => 'BITCOIN GOLD',
                    'priority' => 15,
                    'label' => 'BITCOIN GOLD',
                    'icon' => '/extension/xamax/catalog/view/image/btg.svg'
                ];
            case 'dash':
                return [
                    'network' => 'DASH',
                    'priority' => 16,
                    'label' => 'DASH',
                    'icon' => '/extension/xamax/catalog/view/image/dash.svg'
                ];
            default:
                return [
                    'network' => strtoupper($currency),
                    'priority' => 17,
                    'label' => strtoupper($currency),
                    'icon' => '/extension/xamax/catalog/view/image/default.png'
                ];
        }
    }

    public function checkout()
    {
        $this->setupXamaxClient();
        $this->load->model('checkout/order');
        $this->load->model('extension/xamax/payment/xamax');

        $payment_xamax_receive_currency = $_POST['payment_xamax_receive_currency'] ?? 'btc';

        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $description = [];

        foreach ($this->cart->getProducts() as $product) {
            $description[] = $product['quantity'] . ' × ' . $product['name'];
        }

        $amount = $order_info['total'] * $this->currency->getvalue($order_info['currency_code']);

        $cg_order = Order::create(array(
            'txId' => $order_info['order_id'],
            'code' => mb_strtolower($payment_xamax_receive_currency),
            'fiat' => [
                'amount' => $amount,
                'currency' => 'usd',
            ]
        ));

        if ($cg_order) {
            $this->model_extension_xamax_payment_xamax->addOrder(array(
                'order_id' => $order_info['order_id'],
                'xamax_id' => $cg_order->id,
                'xamax_code' => $cg_order->codeRequested,
                'xamax_wallet' => $cg_order->walletAddress,
                'xamax_amount' => $cg_order->amountRequired,
                'xamax_amount_unit' => $cg_order->amountRequiredUnit,
                'xamax_status' => $cg_order->status
            ));

            $this->model_checkout_order->addHistory($order_info['order_id'], $this->config->get('payment_xamax_order_status_id'));

            $this->response->redirect($this->url->link('extension/xamax/payment/xamax.order', 'order_id=' . $order_info['order_id']));
        } else {
            $this->log->write("Order #" . $order_info['order_id'] . " is not valid. Please check Xamax API request logs.");
            $this->response->redirect($this->url->link('checkout/checkout', '', true));
        }
    }

    public function cron()
    {
        $testConnection = XamaxClient::testConnection(array(
            'environment' => $this->config->get('payment_xamax_test_mode'),
            'refresh_token' => $this->config->get('payment_xamax_api_refresh_token')));
    }

	public function confirm()
	{
		$this->load->language('extension/xamax/payment/xamax');

		$this->load->model('checkout/order');

		$this->model_checkout_order->addHistory($this->session->data['order_id'], (int) $this->config->get('payment_xamax_approved_status_id'), '', true);

		$this->response->redirect(
			$this->url->link('checkout/success', 'language=' . $this->config->get('config_language'), true)
		);
	}

    public function cancel()
    {
        $this->response->redirect($this->url->link('checkout/cart', ''));
    }

    public function order()
    {
        $this->load->language('extension/xamax/payment/xamax');
        $this->load->model('extension/xamax/payment/xamax');

        $order_id = (int)$_GET['order_id'];

        $query = $this->model_extension_xamax_payment_xamax->getOrder($order_id);

        if (isset($query) && count($query)) {
            $data = $query;

            $data['currency_payload'] = $this->getIconAndLabelForCurrency($data['xamax_code']);

            ob_start();

            QRcode::png( $query['xamax_wallet'], null, QR_ECLEVEL_L, 8 );
            $data['qr_code'] = base64_encode( ob_get_contents() );

            ob_end_clean();

            $data['header'] = $this->load->controller('common/header');
            $data['footer'] = $this->load->controller('common/footer');
            $this->response->setOutput($this->load->view('extension/xamax/payment/xamax_order', $data));
        } else {
            $this->response->redirect($this->url->link('common/home', '', true));
        }
    }

    public function webhook()
    {
        $this->load->model('checkout/order');
        $this->load->model('extension/xamax/payment/xamax');

        $order_id = $this->request->post['txId'];
        $order_info = $this->model_checkout_order->getOrder($order_id);
        $ext_order = $this->model_extension_xamax_payment_xamax->getOrder($order_id);


        if (!empty($order_info) && !empty($ext_order)) {
            $this->setupXamaxClient();

            $cg_order = Order::find($order_id);

            if ($cg_order) {
                switch ($cg_order->status) {
                    case 'transaction_status_processing':
                        $cg_order_status = 'payment_xamax_processing_status_id';
                        break;
                    case 'transaction_status_confirmed':
                        $cg_order_status = 'payment_xamax_confirming_status_id';
                        break;
                    case 'transaction_status_failed':
                        $cg_order_status = 'payment_xamax_invalid_status_id';
                        break;
                    case 'transaction_status_expired':
                        $cg_order_status = 'payment_xamax_expired_status_id';
                        break;
                    case 'transaction_status_canceled':
                        $cg_order_status = 'payment_xamax_canceled_status_id';
                        break;
                    case 'transaction_status_refunded':
                        $cg_order_status = 'payment_xamax_refunded_status_id';
                        break;
                    default:
                        $cg_order_status = NULL;
                }

                if (!is_null($cg_order_status)) {
                    $this->model_checkout_order->addOrderHistory($order_id, $this->config->get($cg_order_status));
                }
            }
        }

        $this->response->addHeader('HTTP/1.1 200 OK');
    }

    private function setupXamaxClient()
    {
		XamaxClient::config(array(
            'environment' => $this->config->get('payment_xamax_test_mode') == 1 ? 'sandbox' : 'live',
            'access_token' => empty($this->config->get('payment_xamax_api_access_token')) ? $this->config->get('payment_xamax_api_secret') : $this->config->get('payment_xamax_api_access_token'),
            'refresh_token' => $this->config->get('payment_xamax_api_refresh_token'),
            'user_agent' => 'Xamax - OpenCart v' . VERSION . ' Extension v' . XAMAX_OPENCART_EXTENSION_VERSION
        ));
    }
}
