<?php
namespace Xamax\Merchant;

use Xamax\XamaxClient;
use Xamax\Merchant;
use Xamax\OrderIsNotValid;
use Xamax\OrderNotFound;

class Order extends Merchant
{
    private $order;

    public function __construct($order)
    {
        $this->order = $order;
    }

    public function toHash()
    {
        return $this->order;
    }

    public function __get($name)
    {
        return $this->order[$name];
    }

    public static function find($orderId, $options = array(), $authentication = array())
    {
        try {
            return self::findOrFail($orderId, $options, $authentication);
        } catch (OrderNotFound $e) {
            return false;
        }
    }

    public static function findOrFail($orderId, $options = array(), $authentication = array())
    {
        $authentication = XamaxClient::requestAuth();
        $order = XamaxClient::request('/v1/transaction/incoming/' . $orderId, 'GET', array(), $authentication);
        $order['refresh_token'] = $authentication['refresh_token'];

        return new self($order);
    }

    public static function create($params, $options = array(), $authentication = array())
    {
        try {
            return self::createOrFail($params, $options, $authentication);
        } catch (OrderIsNotValid $e) {
            return false;
        }
    }

    public static function createOrFail($params, $options = array(), $authentication = array())
    {
        $authentication = XamaxClient::requestAuth();
        $order = XamaxClient::request('/v1/transaction/invoice', 'POST', $params, $authentication);
        $order['refresh_token'] = $authentication['refresh_token'];

        return new self($order);
    }

    public static function exchangerate($options = array(), $authentication = array())
    {
        try {
            return self::exchangerateOrFail($options, $authentication);
        } catch (OrderIsNotValid $e) {
            return false;
        }
    }

    public static function exchangerateOrFail($options = array(), $authentication = array())
    {
        $authentication = XamaxClient::requestAuth();
        $exchangerate = XamaxClient::request('/v1/exchangerate', 'GET', array(), $authentication);
        $exchangerate['refresh_token'] = $authentication['refresh_token'];

        return new self($exchangerate);
    }

    public static function chaingtype()
    {
        try {
            return self::chaingtypeOrFail();
        } catch (OrderIsNotValid $e) {
            return false;
        }
    }

    public static function chaingtypeOrFail()
    {
        $chaingtype = XamaxClient::request('/.config/processing.json', 'GET', array(), array('access_token' => 'guest'));

        return new self($chaingtype);
    }
}
