<?php

namespace Xamax;

class XamaxClient
{
    const VERSION = '0.0.0';
    const USER_AGENT_ORIGIN = 'Xamax PHP Library';

    public static $access_token = '';
    public static $refresh_token = '';
    public static $environment = 'live';
    public static $user_agent = '';
    public static $curlopt_ssl_verifypeer = FALSE;

    public static function config($authentication)
    {
        if (isset($authentication['access_token']))
            self::$access_token = $authentication['access_token'];

        if (isset($authentication['refresh_token']))
            self::$refresh_token = $authentication['refresh_token'];

        if (isset($authentication['environment']))
            self::$environment = $authentication['environment'];

        if (isset($authentication['user_agent']))
            self::$user_agent = $authentication['user_agent'];
    }

    public static function testConnection($authentication = array())
    {
        try {
            $response = self::requestAuth($authentication);

            return $response;
        } catch (\Exception $e) {
            return get_class($e) . ': ' . $e->getMessage();
        }
    }

    /**
     * @throws APIError
     * @throws UnprocessableEntity
     * @throws NotFound
     * @throws Unauthorized
     * @throws BadRequest
     * @throws InternalServerError
     * @throws OrderIsNotValid
     * @throws PageNotFound
     * @throws BadEnvironment
     * @throws CredentialsMissing
     * @throws RecordNotFound
     * @throws OrderNotFound
     * @throws RateLimitException
     * @throws BadCredentials
     */
    public static function request($url, $method = 'POST', $params = [], $authentication = [])
    {
        $access_token = $authentication['access_token'] ?? self::$access_token;
        $environment = $authentication['environment'] ?? self::$environment;

        # Check if credentials was passed
        if (empty($access_token))
            Exception::throwException(400, array('reason' => 'AuthTokenMissing'));

        # Check if right environment passed
        $environments = array('live', 'sandbox');

        if (!in_array($environment, $environments)) {
            $availableEnvironments = join(', ', $environments);
            Exception::throwException(400, array('reason' => 'BadEnvironment', 'message' => "Environment does not exist. Available environments: $availableEnvironments"));
        }

        $url = ($environment === 'sandbox' ? 'https://api.sandbox.xamax.io' : 'https://api.xamax.io') . $url;
        $curl = curl_init();

        $curl_options = [
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_URL => $url,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 30,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        ];

        if ($method == 'POST') {
            $curl_options[CURLOPT_POST] = true;
            curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($params));
        }

        curl_setopt_array($curl, $curl_options);

        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            "Authorization: Bearer $access_token",
            "Accept: application/json",
            "Content-Type: application/json"
        ]);

        $response = curl_exec($curl);
        $http_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        $response = json_decode($response, true);

        if ($http_status !== 200) {
            Exception::throwException($http_status, $response);
        }

        return $response;
    }

    public static function requestAuth($authentication = array())
    {
        $refresh_token = $authentication['refresh_token'] ?? self::$refresh_token;
        $environment = $authentication['environment'] ?? self::$environment;
        $user_agent = $authentication['user_agent'] ?? (self::$user_agent ?? (self::USER_AGENT_ORIGIN . ' v' . self::VERSION));
        $curlopt_ssl_verifypeer = $authentication['curlopt_ssl_verifypeer'] ?? self::$curlopt_ssl_verifypeer;

        # Check if right environment passed
        $environments = array('live', 'sandbox');

        if (!in_array($environment, $environments)) {
            $availableEnvironments = join(', ', $environments);
            Exception::throwException(400, array('reason' => 'BadEnvironment', 'message' => "Environment does not exist. Available environments: $availableEnvironments"));
        }

        $url = ($environment === 'sandbox' ? 'https://api.sandbox.xamax.io' : 'https://api.xamax.io') . '/v1/auth/refresh';
        $headers = array();
        $curl = curl_init();

        $curl_options = array(
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_URL => $url
        );

        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        array_merge($curl_options, array(CURLOPT_POST => 1));
        curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query(['refresh_token' => $refresh_token]));

        curl_setopt_array($curl, $curl_options);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_USERAGENT, $user_agent);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, $curlopt_ssl_verifypeer);

        $response = curl_exec($curl);
        $http_status = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        $response = json_decode($response, TRUE);

        if ($http_status === 200)
            return $response;
        else
            Exception::throwException($http_status, $response);
    }
}
